import greenfoot.*;  // (World, Actor, GreenfootImage, Greenfoot and MouseInfo)

/**
 * An enemy-creep-thing.
 * 
 * @author Ninto
 * @version 0.1
 */
public abstract class Enemy  extends Actor
{
    protected int mode, type;
    static final int NORM_MODE = 0, POIS_MODE = 1, FROZ_MODE = 2, BURN_MODE = 3;
    /**This enemys health*/
    int health;
    /**The strength of this enemy*/
    protected int strength;
    /**How much you'll gain when this is killed*/
    private int price;
    private double speed, originalSpeed, move;
    private int mode_delay, healthReg;
    
    private static GreenfootImage[][] img;
    
    /**
     * Create a enemy.
     */
    public Enemy(int strength, int health, double speed, int price) {
        this.strength = strength;
        this.health = health;
        this.speed = speed;
        originalSpeed = speed;
        this.price = price;
        if(img == null)
        img = new GreenfootImage[][] {
            {
                new GreenfootImage("EnemyImages/enemy.png"), new GreenfootImage("EnemyImages/enemyPOIS.png"),
                new GreenfootImage("EnemyImages/enemyFROZ.png"), new GreenfootImage("EnemyImages/enemyBURN.png")
            }, {
                new GreenfootImage("EnemyImages/fastEnemy.png"), new GreenfootImage("EnemyImages/fastEnemyPOIS.png"),
                new GreenfootImage("EnemyImages/fastEnemyFROZ.png"), new GreenfootImage("EnemyImages/fastEnemyBURN.png")
            }, {
                new GreenfootImage("EnemyImages/armorEnemy.png"), new GreenfootImage("EnemyImages/armorEnemyPOIS.png"),
                new GreenfootImage("EnemyImages/armorEnemyFROZ.png"), new GreenfootImage("EnemyImages/armorEnemyBURN.png")
            }, {
                new GreenfootImage("EnemyImages/armorlessEnemy.png"), new GreenfootImage("EnemyImages/armorlessEnemyPOIS.png"),
                new GreenfootImage("EnemyImages/armorlessEnemyFROZ.png"), new GreenfootImage("EnemyImages/armorlessEnemyBURN.png")
            }, {
                new GreenfootImage("EnemyImages/bigEnemy.png"), new GreenfootImage("EnemyImages/bigEnemyPOIS.png"),
                new GreenfootImage("EnemyImages/bigEnemyFROZ.png"), new GreenfootImage("EnemyImages/bigEnemyBURN.png")
            }, {
                new GreenfootImage("EnemyImages/smallEnemy.png"), new GreenfootImage("EnemyImages/smallEnemyPOIS.png"),
                new GreenfootImage("EnemyImages/smallEnemyFROZ.png"), new GreenfootImage("EnemyImages/smallEnemyBURN.png")
            }, {
                new GreenfootImage("EnemyImages/tinyEnemy.png"), new GreenfootImage("EnemyImages/tinyEnemyPOIS.png"),
                new GreenfootImage("EnemyImages/tinyEnemyFROZ.png"), new GreenfootImage("EnemyImages/tinyEnemyBURN.png")
            }
        };
        if(this instanceof NormalEnemy) type = 0;
        if(this instanceof SpeedEnemy) type = 1;
        if(this instanceof ShellBug) type = 2;
        if(this instanceof BigEnemy) type = 4;
        if(this instanceof SmallEnemy) type = 5;
        if(this instanceof TinyEnemy) type = 6;
        setImage();
    }
    
    /**
     * Very acty...
     */
    public void act() {
        if(mode_delay == 0) setMode(NORM_MODE);
        else if(mode_delay > 0) mode_delay--;
        if(myWorld.randomChance(10)) health += healthReg;
        run();
        Castle target = (Castle)getOneIntersectingObject(Castle.class);
        if(target != null) {
            target.health -= strength;
            getWorld().removeObject(this);
            return;
        }
        if(health <= 0) die();
    }
    
    /**Check if this can move*/
    public boolean canMoveTo(int x, int y) {
        return !getWorld().getObjectsAt(x, y, Path.class).isEmpty();
    }
    
    /**Move*/
    protected void move() {
        double angle = Math.toRadians(getRotation());
        int x = getX()+(int)Math.round(Math.cos(angle));
        int y = getY()+(int)Math.round(Math.sin(angle));
        if(canMoveTo(x, y)) setLocation(x, y);
    }
    
    /**Return a myWorld*/
    public myWorld getWorld() {
        return (myWorld)super.getWorld();
    }
    
    public void run() {
        move += speed;
        while(move >= 1) {
            move();
            turn();
            move--;
        }
    }
    
    public void turn() {
        Path path = (Path)getOneIntersectingObject(Path.class);
        if(path == null || path.getDirection() == -1 || path.getX() != getX() || path.getY() != getY()) return;
        int rot = getRotation();
        switch(path.getDirection()) {
            case 0:
            if(rot == 0) setRotation(270);
            else if(rot == 90) setRotation(180);
            break;
            case 1:
            if(rot == 180) setRotation(270);
            else if(rot == 90) setRotation(0);
            break;
            case 2:
            if(rot == 180) setRotation(90);
            else if(rot == 270) setRotation(0);
            break;
            case 3:
            if(rot == 0) setRotation(90);
            else if(rot == 270) setRotation(180);
            break;
        }
    }
    
    protected void die() {
        getWorld().setMoney(getWorld().getMoney()+price);
        getWorld().removeObject(this);
        Greenfoot.playSound("die.wav");
    }
    
    protected void setImage() {
        setImage(img[type][mode]);
    }
    
    public void setMode(int mode) {
        this.mode = mode;
        setImage();
        checkMode();
    }
    
    private void checkMode() {
        switch(mode) {
            case NORM_MODE:
            speed = originalSpeed;
            healthReg = 0;
            mode_delay = -1;
            break;
            case POIS_MODE:
            speed = originalSpeed/1.5;
            healthReg = -1;
            mode_delay = 64;
            break;
            case FROZ_MODE:
            speed = 0;
            healthReg = 0;
            mode_delay = 80;
            break;
            case BURN_MODE:
            speed = originalSpeed*1.2;
            healthReg = -2;
            mode_delay = 64;
            break;
        }
    }
}