import greenfoot.*;  // (World, Actor, GreenfootImage, Greenfoot and MouseInfo)

public abstract class Turret  extends GridActor
{
    /**This' name*/
    private String name;
    /**This is placed*/
    private boolean placed;
    /**This is selected*/
    private boolean selected = true;
    /**This is <i>just</i> selected.*/
    private boolean justSelected = true;
    protected int delay;
    protected int rate = 32;
    protected int attack = 1;
    protected int range = 100;
    protected int price = 10;
    protected int level = 1;
    protected int gain = 2;
    
    /**Image, the*/
    private GreenfootImage img;
    /**Static image, the*/
    protected GreenfootImage staticImg = new GreenfootImage("bulletTurret.png");
    
    /**
     * Create a bulletTurret.
     */
    public Turret() {
        super(32);
        if(this instanceof BulletTurret) name = "Bullet";
        if(this instanceof BombTurret) name = "Bomb";
        if(this instanceof PoisonTurret) name = "Poison";
        if(this instanceof LaserTurret) name = "Laser";
        if(this instanceof FlameTurret) name = "Flame";
        if(this instanceof FrozenTurret) name = "Frozen";
    }
    
    /**
     * Addy. \:
     */
    protected void addedToWorld(World world) {
        updateImage();
    }
    
    /**
     * Acty again.
     */
    public void act() {
        if(!placed) {
            setLocation(myWorld.mx, myWorld.my);
            if(Greenfoot.isKeyDown("escape")) {
                getWorld().removeObject(this);
                return;
            }
            if(Greenfoot.mousePressed(this) && canPlace()) {
                placed = true;
                getWorld().setMoney(getWorld().getMoney()-price);
                grid();
            }
        } else {
            if(Greenfoot.mousePressed(this)) {selected = true; justSelected = true;}
            else if(Greenfoot.mousePressed(null)) {selected = false; justSelected = true;}
            if(selected) {
                if(justSelected) {
                    setImage(img);
                    getWorld().uploadStats(name, attack, attack+gain/2, rate,
                    rate-rate/(gain*3), range, range+gain*5, level*price/2, level, this);
                    justSelected = false;
                }
            } else if(justSelected) {
                setImage(staticImg);
                if(getWorld().getSelectedTurret() == this) getWorld().reset();
                justSelected = false;
            }
            
            aimAndShoot();
        }
    }
    
    /**
     * Updaty imagy.
     */
    public void updateImage() {
        img = new GreenfootImage(range*2, range*2);
        img.setColor(new java.awt.Color(255, 0, 0, 100));
        img.fillOval(0, 0, range*2-1, range*2-1);
        img.setColor(java.awt.Color.RED);
        img.drawOval(0, 0, range*2-1, range*2-1);
        
        img.drawImage(staticImg, range-staticImg.getWidth()/2, range-staticImg.getHeight()/2);
        
        setImage(img);
    }
    
    /**
     * Aimy shooty.
     */
    private void aimAndShoot() {
        java.util.List<Enemy> enemies = getObjectsInRange(range, Enemy.class);
        if(!enemies.isEmpty()) {
            Enemy target = enemies.get(0);
            setRotation((int)Math.toDegrees(Math.atan2(target.getY()-getY(), target.getX()-getX())));
        }
        if(delay > 0) {delay--; return;}
        if(!enemies.isEmpty())shootBullet();
        delay = rate;
    }
    
    /**
     * Shooty bullety.
     */
    protected abstract void shootBullet();
    
    /**
     * Cany placy.
     */
    private boolean canPlace() {
        int x = getGridX();
        int y = getGridY();
        return getWorld().getObjectsAt(x, y, GridActor.class).size() <= 1;
    }
    
    /**
     * Sety statsy.
     */
    public void setStats(int atk, int rate, int range) {
        level++;
        attack = atk;
        this.rate = rate;
        this.range = range;
        updateImage();
    }
}